package no.usit.fridaws;

import java.util.ArrayList;

import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.Response;

import no.usit.fridaws.exception.NotFoundException;
import no.usit.fridaws.exception.TooManyException;

import org.jboss.resteasy.annotations.cache.Cache;

/**
 * Person RPC-tjeneste.
 */
@Path("/hentPerson")
public class Person {

    // private static final Logger logger =
    // LoggerFactory.getLogger(Person.class.toString());

    /**
     * Returnerer en person basert paa personens loepenummer, eller brukernavn
     * og eierkode.
     * 
     * @param navn
     *            personens brukernavn
     * @param eierkode
     *            stedet brukernavnet hører hjemme
     * @param lopenr
     *            personens unike loepenummer i databasen
     * @param format
     *            "xml" eller "json" for å angi output format
     * @return en person ressurs
     */
    @GET
    @Cache(maxAge = 0)
    public Response hentPerson(@QueryParam("eierkode") String eierkode, @QueryParam("navn") String navn,
            @QueryParam("lopenr") Long lopenr, @QueryParam("format") String format) {
        if (format == null)
            format = "xml";
        else
            format = format.toLowerCase();
        if (eierkode != null)
            eierkode = eierkode.toUpperCase();

        if (!InputValidering.okPerson(navn, eierkode, lopenr) || !(format.equals("xml") ^ format.equals("json"))) {
            return Response.status(400).build();
        }

        ArrayList<Object> params = new ArrayList<Object>();
        String sql, sFields;

        if (navn != null && eierkode != null) {
            lopenr = Database.getPersonLopeNr(navn, eierkode);
        }

        if (lopenr == -1) {
            return Response.status(500).build();
        }

        if (lopenr == null) {
            return Response.status(404).build();
        }

        sFields = "tekst_xml";
        sql = "SELECT %SFIELDS% " + "FROM   presentasjon_xml x, person p "
                + "WHERE  x.presentasjonslopenr = p.presentasjonslopenr " + "  AND  p.personlopenr = :1 "
                + "  AND  tekst_xml IS NOT null";

        params.add(lopenr);

        try {
            ResponseStreamer stream = new ResponseStreamer("http://frida.usit.uio.no/" + "xmlSchema/xsd/frida.xsd",
                    "forskerprofil", sql, sFields, params, format);
            return Xml.genererXML(format, stream);
        } catch (TooManyException e) {
            return Response.status(413).build();
        } catch (NotFoundException e) {
            return Response.status(404).build();
        } catch (Exception e) {
            return Response.status(500).build();
        }

    }
}
