package no.usit.fridaws;

import java.sql.Blob;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;

import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.Response;

import org.jboss.resteasy.annotations.cache.Cache;

/**
 * PersonBilde RPC-tjeneste.
 */
@Path("/hentPersonBilde")
public class PersonBilde {

    /**
     * Returnerer en persons bilde
     * 
     * @param navn
     *            personens brukernavn
     * @param eierkode
     *            stedet brukernavnet hører hjemme
     * @param lopenr
     *            personens unike loepenummer i databasen
     * @return en persons bilde
     */
    @GET
    @Cache(maxAge = 0)
    @Produces("image/jpeg")
    public Response hentPersonBilde(@QueryParam("eierkode") String eierkode, @QueryParam("navn") String navn,
            @QueryParam("lopenr") Long lopenr) {
        if (eierkode != null)
            eierkode = eierkode.toUpperCase();

        if (!InputValidering.okPerson(navn, eierkode, lopenr)) {
            return Response.status(400).build();
        }

        Blob output = null;
        Connection conn = null;

        try {
            ArrayList<Object> params = new ArrayList<Object>();
            conn = Database.getConnection();
            String sql;

            if (navn != null && eierkode != null)
                lopenr = Database.getPersonLopeNr(navn, eierkode);

            if (lopenr == -1) {
                return Response.status(500).build();
            }
            if (lopenr == null) {
                return Response.status(404).build();
            }
            sql = "SELECT bilde " + "FROM   person " + "WHERE  personlopenr = :1";
            params.add(lopenr);

            ResultSet rs = Database.executeSql(conn, sql, params);

            if (rs.next()) {
                output = rs.getBlob(1);
                if (output == null) {
                    return Response.status(404).build();
                }

            } else {
                return Response.status(404).build();
            }

        } catch (SQLException e) {
            Database.loggFeil(e);
            return Response.status(500).build();

        } finally {
            try {
                if (conn != null) {
                    conn.close();
                }
            } catch (SQLException e) {
                Database.loggFeil(e);
                return Response.status(500).build();
            }
        }

        try {
            return Response.status(200).entity(output.getBinaryStream()).build();

        } catch (SQLException e) {
            Database.loggFeil(e);
            return Response.status(500).build();
        }
    }

}
