package no.usit.fridaws;

import java.util.ArrayList;

import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.Response;

import no.usit.fridaws.exception.NotFoundException;
import no.usit.fridaws.exception.TooManyException;

import org.jboss.resteasy.annotations.cache.Cache;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * PersonSted RPC-tjeneste.
 */
@Path("/hentPersonSted")
public class PersonSted {

    private static final Logger logger = LoggerFactory.getLogger(Database.class);


    /**
     * Returnerer personene som er knyttet til et sted.
     * 
     * @param instnr
     *            stedets institusjonsnr.
     * @param avdnr
     *            stedets avdelingsnr.
     * @param undavdnr
     *            stedets underavdelingsnr.
     * @param gruppenr
     *            stedets gruppenr.
     * @param sortering
     *            sorteringsangivelse, se bruker-doc.
     * @param format
     *            "xml" eller "json" for å angi output format
     * @return en liste med personer
     */
    @GET
    @Cache(maxAge = 0)
    public Response hentPersonSted(@QueryParam("instnr") Long instnr,
            @QueryParam("avdnr") Long avdnr,
            @QueryParam("undavdnr") Long undavdnr,
            @QueryParam("gruppenr") Long gruppenr,
            @QueryParam("format") String format,
            @QueryParam("sortering") String sortering) {

        if (format == null) {
            format = "xml";
        } else {
            format = format.toLowerCase();
        }

        if ((instnr == null) || (!format.equals("xml") && !format.equals("json"))) {
            return Response.status(400).build();
        }

        ArrayList<Object> params = new ArrayList<Object>();
        String sql, sFields;

        sFields = "x.tekst_xml";
        sql = "SELECT %SFIELDS% from " + "       presentasjon_xml x, "
              + "       presentasjon_enhet pe, "
              + "       presentasjon p "
              + "WHERE  pe.presentasjonslopenr = p.presentasjonslopenr "
              + "  AND  p.presentasjonslopenr = x.presentasjonslopenr "
              + "  AND  p.presentasjonstypekode = 'PERSON' "
              + "  AND  pe.institusjonsnr = ? ";
        params.add(instnr);

        if (avdnr != null) {
            sql += " AND avdnr = ? ";
            params.add(avdnr);
        }
        if (undavdnr != null) {
            sql += " AND undavdnr = ? ";
            params.add(undavdnr);
        }
        if (gruppenr != null) {
            sql += " AND gruppenr = ? ";
            params.add(gruppenr);
        }

        if (sortering != null) {
            String sortpred = Filter.lagPersonSortering(sortering);
            if (sortpred == null) {
                return Response.status(400).build();
            }
            sql += " ORDER BY " + sortpred + " ";
        }

        // til slutt, begrens antall: (dette maa gjoeres sist)
        // FEILSITUASJON - returnerer 413 uansett om det er for mange
        // sql = "SELECT tekst_xml FROM (" + sql + ") WHERE ROWNUM <= 1001";

        try {
            ResponseStreamer stream = new ResponseStreamer("http://frida.usit.uio.no/" + "xmlSchema/xsd/frida.xsd",
                                                           "forskerprofil", sql, sFields, params, format);
            return Xml.genererXML(format, stream);
        } catch (TooManyException e) {
            return Response.status(413).build();
        } catch (NotFoundException e) {
            return Response.status(404).build();
        } catch (Exception e) {
            return Response.status(500).build();
        }

    }

}
