package no.usit.fridaws;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;

import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.Variant;

import no.usit.fridaws.exception.NotFoundException;
import no.usit.fridaws.exception.TooManyException;

import org.jboss.resteasy.annotations.cache.Cache;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Person RPC-tjeneste.
 */
@Path("/hentPersoner")
public class Personer {
	private static final Logger logger = LoggerFactory.getLogger(Personer.class
			.toString());

	// private static final Logger logger =
	// LoggerFactory.getLogger(Person.class.toString());

	/**
	 * Returnerer en personer basert paa personens loepenummer, eierkode og evt.
	 * sist endret dato
	 * 
	 * @param eierkode
	 *            stedet brukerne hører hjemme
	 * @param format
	 *            "xml" eller "json" for å angi output format
	 * @return en liste av personer
	 */
	@GET
	@Cache(maxAge = 0)
	public Response hentPerson(@QueryParam("eierkode") String eierkode,
			@QueryParam("format") String format,
			@QueryParam("endretsiden") String endretSiden) {
		String endretSidenSql = null;
		if (format == null)
			format = "xml";
		else
			format = format.toLowerCase();
		if (eierkode != null)
			eierkode = eierkode.toUpperCase();
		if (endretSiden != null) {
			endretSidenSql = "AND (to_date( :2,'ddmmyyyy')<DATO_SISTE_ENDRING OR  to_date( :2,'ddmmyyyy')<DATO_SISTE_ENDRING_VARBEID)";

		}

		String sql = "SELECT personlopenr, b.brukernavn "
				+ " FROM person p, bruker b "
				+ " WHERE p.fodselsdato = b.fodselsdato AND  p.personnr = b.personnr and b.eierkode= :1 AND p.status_dod = 'N' and p.status_sperret='N' "
				+ endretSidenSql;
		logger.info(sql);
		StringBuffer output = new StringBuffer(1000);
		MediaType outputFormat = MediaType.APPLICATION_XML_TYPE;
		if (format.equals("json")) {
			outputFormat = MediaType.APPLICATION_JSON_TYPE;
		}
		output.append("<personer>");
		Connection conn = null;
		String lopenr = null;
		try {
			conn = Database.getConnection();
			PreparedStatement stmt = null;
			stmt = conn.prepareStatement(sql);
			stmt.setString(1, eierkode);
			if (endretSiden != null) {
				stmt.setString(2, endretSiden);
				stmt.setString(3, endretSiden);
			}
			ResultSet rs = stmt.executeQuery();
			while (rs.next()) {
				output.append("<person>\n");
				lopenr = rs.getString(1);
				output.append("<personlopenr>" + rs.getString(1)
						+ "</personlopenr>\n");
				output.append("<brukernavn>" + rs.getString(2)
						+ "</brukernavn>\n");

				output.append("</person>\n");
			}

		} catch (SQLException e) {
			logger.info("Feil ved lopenr " + lopenr);
			Database.loggFeil(e);
		} finally {
			try {
				if (conn != null)
					conn.close();
			} catch (SQLException e) {
				Database.loggFeil(e);

			}
		}

		output.append("</personer>");

		return Response
				.status(200)
				.entity(output.toString())
				.variant(
						new Variant(outputFormat,
								new java.util.Locale("no_NO"), null)).build();
	}
}
