package no.usit.fridaws;

import java.util.ArrayList;

import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.Response;

import no.usit.fridaws.exception.NotFoundException;
import no.usit.fridaws.exception.TooManyException;

import org.jboss.resteasy.annotations.cache.Cache;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Presentasjon RPC-tjeneste.
 */
@Path("/hentPresentasjon")
public class Presentasjon {

    private static final Logger logger = LoggerFactory.getLogger(Presentasjon.class.toString());


    /**
     * Returnerer en presentasjon, gitt et loepenr. Hvis type er oppgitt, og presentasjonen ikke er av denne typen,
     * returneres en 404 respons.
     * 
     * @param lopenr
     *            presentasjonens unike loepenummer i databasen
     * @param type
     *            presentasjonstype
     * @param format
     *            "xml" eller "json" for å angi output format
     * @return en presentasjon
     */
    @GET
    @Cache(maxAge = 0)
    public Response hentPerson(@QueryParam("lopenr") Long lopenr,
            @QueryParam("type") String type,
            @QueryParam("format") String format) {
        if (format == null) {
            format = "xml";
        } else {
            format = format.toLowerCase();
        }
        if (type != null) {
            type = type.toUpperCase();
        }

        if (lopenr == null || !(format.equals("xml") ^ format.equals("json"))) {
            return Response.status(400).build();
        }

        ArrayList<Object> params = new ArrayList<Object>();
        String sql, sFields;

        sFields = "tekst_xml";
        sql = "SELECT %SFIELDS% " + "FROM   presentasjon_xml x, "
              + "       presentasjon p "
              + "WHERE  p.presentasjonslopenr = x.presentasjonslopenr "
              + "  AND  p.presentasjonslopenr = ? ";

        if (type != null) {
            sql += " AND presentasjonstypekode = ? ";
        }

        params.add(lopenr);
        if (type != null) {
            params.add(type);
        }

        logger.debug(sql);

        try {
            ResponseStreamer stream = new ResponseStreamer("http://frida.usit.uio.no/" + "xmlSchema/xsd/frida.xsd",
                                                           "forskningsenhet|forskerprofil|prosjekt", sql, sFields,
                                                           params, format);
            return Xml.genererXML(format, stream);
        } catch (TooManyException e) {
            return Response.status(413).build();
        } catch (NotFoundException e) {
            return Response.status(404).build();
        } catch (Exception e) {
            return Response.status(500).build();
        }

    }

}
