package no.usit.fridaws;

import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.StreamingOutput;

import net.sf.json.xml.XMLSerializer;
import no.usit.fridaws.exception.NotFoundException;
import no.usit.fridaws.exception.TooManyException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class ResponseStreamer implements StreamingOutput {

    private static int MAX_ROWS = 1000;
    private static long MAX_BYTES = 50000000;

    private String dato;
    String skjemaURI;
    String elementToExtract;
    String sql;
    String format = "XML";
    ArrayList<Object> params;
    ResultSet rs;
    private Connection conn;
    private static final Logger logger = LoggerFactory.getLogger(Database.class);


    public ResponseStreamer(String skjemaURI,
            String elementToExtract,
            String sql,
            String sFields,
            ArrayList<Object> params,
            String format) throws NotFoundException, TooManyException {
        super();

        this.skjemaURI = skjemaURI;
        this.elementToExtract = elementToExtract;
        this.params = params;
        this.format = format;

        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        setDato(sdf.format(new Date()));

        conn = getConnection();

        String cSql = new String(sql);
        cSql = cSql.replaceAll("%SFIELDS%", "COUNT(*), SUM(LENGTH(" + sFields + "))");
        // cSql = cSql.replaceAll("ORDER BY.*$", "");
        sql = sql.replaceAll("%SFIELDS%", sFields);
        this.sql = sql;

        try {
            ResultSet rs = Database.executeSql(conn, cSql, params);
            if (rs.next()) {
                int rows = new Integer(rs.getString(1)).intValue();
                if (rows == 0) {
                    logger.debug("Ingen treff: " + sql + "(" + params + ")");
                    closeConnection();
                    throw new NotFoundException();
                }
                long size = new Long(rs.getString(2)).longValue();
                if (rows > MAX_ROWS || size > MAX_BYTES) {
                    logger.debug("For mye data (" + rows + " treff, " + size + " bytes): " + sql + "(" + params + ")");
                    closeConnection();
                    throw new TooManyException();
                }
                logger.debug("Fikk " + rows + " treff: " + sql + "(" + params + ")");
            }
        } catch (SQLException e) {
            logger.error("###### Could not check number of rows returned, panic! " + cSql
                         + " (params: "
                         + params
                         + ", sFields: "
                         + sFields
                         + ")");
            e.printStackTrace();
        }

    }


    @Override
    public void write(OutputStream outStream) throws IOException, WebApplicationException {
        // System.out.println("#### Format: " + format);
        XMLSerializer xs = new XMLSerializer();
        OutputStreamWriter out = new OutputStreamWriter(outStream);
        if (format.equals("json")) {
            out.write("{\"@xmlns:xsi\":\"http://www.w3.org/2001/XMLSchema-instance\",\"@xsi:noNamespaceSchemaLocation\":\"" + skjemaURI
                      + "\",\"generert\":\""
                      + dato
                      + "\",\"forskningsresultat\":[");
        } else {
            out.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?><frida xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xsi:noNamespaceSchemaLocation=\"" + skjemaURI
                      + "\">");
            out.write("<generert>" + getDato() + "</generert>");
        }

        int i = 0;
        try {
            conn = getConnection();
            rs = Database.executeSql(conn, sql, params);

            /*
            if (!rs.next()) {
                System.out.println("Not found");
                feil(out, "Ikke funnet");
                conn.close();
                throw new NotFoundException();
            }

            rs.last();
            int rows = rs.getRow();
            if (rows > 1000) {
                System.out.println("Not found");
                feil(out, "For mange treff");
                conn.close();
                throw new TooManyException();
            }

            System.out.println("##### rows = " + rows);
            */

            rs.first();
            do {
                // parts.add(rs.getString(1));
                String part;
                part = rs.getString(1);
                Pattern p = Pattern.compile("(<(" + elementToExtract + ")>.*</(" + elementToExtract + ")>)",
                                            Pattern.CASE_INSENSITIVE);
                Matcher m = p.matcher(part);
                if (m.find()) {
                    if (format.equals("json")) {
                        if (i > 0) {
                            out.write(",");
                        }
                        String output = xs.read(m.group(0)).toString();
                        out.write(output);
                    } else {
                        out.write(m.group(0));
                    }
                    // System.out.println("Added " + i++);
                    i++;
                }
            } while (rs.next());

            conn.close();

        } catch (Exception e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } finally {
            try {
                rs.close();
            } catch (SQLException e) {
            }
        }

        if (format.equals("json")) {
            out.write("]}");
        } else {
            out.write("</frida>");
        }
        out.flush();
        // System.out.println("Done with adding " + i);

    }


    public Connection getConnection() {
        if (conn == null) {
            try {
                conn = Database.getConnection();
            } catch (SQLException e) {
                e.printStackTrace();
            }
        }
        return conn;
    }


    public void closeConnection() {
        try {
            conn.close();
            conn = null;
        } catch (SQLException e) {
        }
    }


    /*
    public void feil(OutputStreamWriter out, String mld) {
        mld = "<feilmelding>" + mld + "</feilmelding>";
        if (format.equals("json")) {
            XMLSerializer xs = new XMLSerializer();
            mld = xs.read(mld).toString();
        }
        try {
            out.write(mld);
        } catch (IOException e) {
        }
    }
    */

    public void setDato(String dato) {
        this.dato = dato;
    }


    public String getDato() {
        return dato;
    }

}
