package no.usit.fridaws;

import java.io.StringWriter;
import java.sql.ResultSet;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.StreamingOutput;
import javax.ws.rs.core.Variant;
import javax.xml.transform.Result;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import no.usit.fridaws.exception.NotFoundException;
import no.usit.fridaws.exception.TooManyException;

import org.w3c.dom.Document;

/**
 * Klasse for håndtering av XML.
 * 
 */
public class Xml {

    /**
     * Exception som kan oppstå i forbindelse med håndtering av XML.
     * 
     */
    public static class XmlException extends Exception {

        private static final long serialVersionUID = 1L;


        XmlException(String msg, Throwable ex) {
            super(msg, ex);
        }
    }


    /**
     * Lager XPathExpression fra en xpath spesifisert gjennom en streng.
     * 
     * @param path
     *            streng som representerer en XPath
     * @return et XPathExpression objekt
     */
    public static XPathExpression stringToXPath(String path) throws XPathExpressionException {
        return XPathFactory.newInstance().newXPath().compile(path);
    }


    public static Response genererXML(String format, String output) {
        // Hele metoden må fjernes etterhvert
        return null;
    }


    public static String join(String skjemaURI, String elementToExtract, List<String> parts) throws XmlException {
        // Hele metoden må fjernes etterhvert
        return "";
    }


    public static String join(String skjemaURI, String elementToExtract, ResultSet rs) throws XmlException {

        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        String dato = sdf.format(new Date());

        return joinMedDato(dato, skjemaURI, elementToExtract, rs);

    }


    /**
     * Syr sammen en liste av XML-biter til et dokument.
     * 
     * @param skjemaURI
     *            URI til skjema for resultat-XML
     * @param partsPath
     *            biter med XML
     * @return sammensatt XML
     */
    public static String
            joinMedDato(String dato, String skjemaURI, String elementToExtract, ResultSet rs) throws XmlException {

        return "";

        /*
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        dbf.setIgnoringElementContentWhitespace(true);

        DocumentBuilder docBuilder;
        try {
            docBuilder = dbf.newDocumentBuilder();
        } catch (ParserConfigurationException e) {
            throw new XmlException("Kan ikke lage DocumentBuilder", e);
        }

        Document doc = docBuilder.newDocument();

        Element result = doc.createElement("frida");
        result.setAttribute("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance");
        result.setAttribute("xsi:noNamespaceSchemaLocation", skjemaURI);
        doc.appendChild(result);

        Element generert = doc.createElement("generert");

        generert.appendChild(doc.createTextNode(dato));
        result.appendChild(generert);

        String retTreeS = "<frida>";
        for (String part : parts) {
            Document merge = null;
            try {
                merge = docBuilder.parse(new InputSource(new StringReader(part)));
            } catch (Exception e) {
                throw new XmlException("Ugyldig XML funnet", e);
            }

            Node tree = null;
            try {
                tree = (Node) partsPath.evaluate(merge, XPathConstants.NODE);
            } catch (XPathExpressionException e) {
                throw new XmlException("Kan ikke finne angitte eleent", e);
            }
            tree = doc.importNode(tree, true);

            System.out.println("### Tree: " + tree);

            // result.appendChild(tree);
        }
        retTreeS += "</frida>";


        doc.setXmlStandalone(true);
        return docToString(doc);
        */
    }


    /**
     * Lager String av XML Document.
     * 
     * @param doc
     *            et XML Document
     * @return dokumentet som en String
     */
    private static String docToString(Document doc) throws XmlException {
        try {
            Transformer tf = TransformerFactory.newInstance().newTransformer();
            StringWriter sw = new StringWriter();
            DOMSource source = new DOMSource(doc);
            Result result = new StreamResult(sw);
            tf.transform(source, result);
            return sw.toString();
        } catch (Exception e) {
            throw new XmlException("Kunne ikke konv. Document til streng", e);
        }
    }


    /**
     * Outputter XML på slutten
     */
    public static Response genererXML(String format, StreamingOutput output) throws NotFoundException, TooManyException {
        MediaType outputFormat = MediaType.APPLICATION_XML_TYPE;

        if (format.equals("json")) {
            // XMLSerializer xs = new XMLSerializer();
            // output = xs.read("").toString();
            outputFormat = MediaType.APPLICATION_JSON_TYPE;
        }

        Response r = Response.status(200).entity(output)
                             .variant(new Variant(outputFormat, new java.util.Locale("no_NO"), null)).build();
        // System.out.println("Generated response, sending it");
        return r;

    }

}