package no.usit.fridaws.test;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;

import org.junit.Assert;
import org.junit.Test;

public class ITWS {

	String urlPrefix = "http://localhost:8080/ws/";

	private String getRespText(HttpURLConnection connection) throws Exception {
		BufferedReader reader = new BufferedReader(new InputStreamReader(
				connection.getInputStream()));
		String line = reader.readLine();
		String output = line;
		while (line != null) {
			line = reader.readLine();
			output += line;
		}
		return output;
	}

	/*
	 * hentPerson
	 */

	@Test
	public void testHentPerson_lopenr_200() throws Exception {
		// Arrange
		URL url = new URL(urlPrefix + "hentPerson?lopenr=24679");
		HttpURLConnection connection = (HttpURLConnection) url.openConnection();
		connection.setRequestMethod("GET");

		// Act
		int res = connection.getResponseCode();
		String resp = getRespText(connection);
		connection.disconnect();

		// Assert
		Assert.assertEquals(200, res);
		Assert.assertTrue(resp.contains("Fennefoss"));
	}

	@Test
	public void testHentPerson_lopenr_404() throws Exception {
		// Arrange
		URL url = new URL(urlPrefix + "hentPerson?lopenr=246790");
		HttpURLConnection connection = (HttpURLConnection) url.openConnection();
		connection.setRequestMethod("GET");

		// Act
		int res = connection.getResponseCode();
		connection.disconnect();

		// Assert
		Assert.assertEquals(404, res);
	}

	@Test
	public void testHentPerson_navnOgEierkode_200() throws Exception {
		// Arrange
		URL url = new URL(urlPrefix + "hentPerson?navn=aal041&eierkode=UIB");
		HttpURLConnection connection = (HttpURLConnection) url.openConnection();
		connection.setRequestMethod("GET");

		// Act
		int res = connection.getResponseCode();
		String resp = getRespText(connection);
		connection.disconnect();

		// Assert
		Assert.assertEquals(200, res);
		Assert.assertTrue(resp.contains("Alvarez"));
	}

	/*
	 * hentPersonSted
	 */

	@Test
	public void testHentPersonSted_instnr_413() throws Exception {
		// Arrange
		URL url = new URL(urlPrefix + "hentPersonSted?instnr=185");
		HttpURLConnection connection = (HttpURLConnection) url.openConnection();
		connection.setRequestMethod("GET");

		// Act
		int res = connection.getResponseCode();
		connection.disconnect();

		// Assert
		Assert.assertEquals(413, res);
	}

	@Test
	public void testHentPersonSted_instnrOgAvdnrOgUndavdnrOgGruppenr_200()
			throws Exception {
		// Arrange
		URL url = new URL(urlPrefix
				+ "hentPersonSted?instnr=185&avdnr=14&undavdnr=3&gruppenr=0");
		HttpURLConnection connection = (HttpURLConnection) url.openConnection();
		connection.setRequestMethod("GET");

		// Act
		int res = connection.getResponseCode();
		String resp = getRespText(connection);
		connection.disconnect();

		// Assert
		Assert.assertEquals(200, res);
		Assert.assertTrue(resp.contains("Galaaen"));
	}

	@Test
	public void testHentPersonSted_instnrOgAvdnrOgUndavdnr_200()
			throws Exception {
		// Arrange
		URL url = new URL(urlPrefix
				+ "hentPersonSted?instnr=185&avdnr=14&undavdnr=3");
		HttpURLConnection connection = (HttpURLConnection) url.openConnection();
		connection.setRequestMethod("GET");

		// Act
		int res = connection.getResponseCode();
		String resp = getRespText(connection);
		connection.disconnect();

		// Assert
		Assert.assertEquals(200, res);
		Assert.assertTrue(resp.contains("Galaaen"));
	}

	/*
	 * hentPersonBilde
	 */

	@Test
	public void testHentPersonBilde_lopenr_200() throws Exception {
		// Arrange
		URL url = new URL(urlPrefix + "hentPersonBilde?lopenr=24679");
		HttpURLConnection connection = (HttpURLConnection) url.openConnection();
		connection.setRequestMethod("GET");

		// Act
		int res = connection.getResponseCode();
		getRespText(connection);
		connection.disconnect();

		// Assert
		Assert.assertEquals(200, res);
	}

	@Test
	public void testHentPersonBilde_navnOgEierkode_200() throws Exception {
		// Arrange
		URL url = new URL(urlPrefix
				+ "hentPersonBilde?navn=nilsst&eierkode=UIO");
		HttpURLConnection connection = (HttpURLConnection) url.openConnection();
		connection.setRequestMethod("GET");

		// Act
		int res = connection.getResponseCode();
		getRespText(connection);
		connection.disconnect();

		// Assert
		Assert.assertEquals(200, res);
	}

	@Test
	public void testHentPersonBilde_lopenr_404() throws Exception {
		// Arrange
		URL url = new URL(urlPrefix + "hentPersonBilde?lopenr=105660");
		HttpURLConnection connection = (HttpURLConnection) url.openConnection();
		connection.setRequestMethod("GET");

		// Act
		int res = connection.getResponseCode();
		connection.disconnect();

		// Assert
		Assert.assertEquals(404, res);
	}

	/*
	 * hentVarbeiderPerson
	 */

	@Test
	public void testHentVarbeiderPerson_navnOgEierkodeOgFraOgTil_200()
			throws Exception {
		// Arrange
		URL url = new URL(
				urlPrefix
						+ "hentVarbeiderPerson?navn=nilsst&eierkode=UIO&fra=2008&til=2010");
		HttpURLConnection connection = (HttpURLConnection) url.openConnection();
		connection.setRequestMethod("GET");

		// Act
		int res = connection.getResponseCode();
		String resp = getRespText(connection);
		connection.disconnect();

		// Assert
		Assert.assertEquals(200, res);
		Assert.assertTrue(resp.contains("2009-02-20T16:08:24Z"));
	}

	@Test
	public void testHentVarbeiderPerson_navnOgEierkodeOgFraOgTilOgHovedkategori_200()
			throws Exception {
		// Arrange
		URL url = new URL(
				urlPrefix
						+ "hentVarbeiderPerson?navn=nilsst&eierkode=UIO&fra=2004&til=2010&hovedkategori=BOK");
		HttpURLConnection connection = (HttpURLConnection) url.openConnection();
		connection.setRequestMethod("GET");

		// Act
		int res = connection.getResponseCode();
		String resp = getRespText(connection);
		connection.disconnect();

		// Assert
		Assert.assertEquals(200, res);
		Assert.assertTrue(resp.contains("2005-02-10T09:43:58Z"));
	}

	@Test
	public void testHentVarbeiderPerson_lopenr_200() throws Exception {
		// Arrange
		URL url = new URL(urlPrefix + "hentVarbeiderPerson?lopenr=15090");
		HttpURLConnection connection = (HttpURLConnection) url.openConnection();
		connection.setRequestMethod("GET");

		// Act
		int res = connection.getResponseCode();
		String resp = getRespText(connection);
		connection.disconnect();

		// Assert
		Assert.assertEquals(200, res);
		Assert.assertTrue(resp.contains("2005-01-10T18:15:43Z"));
	}

	/*
	 * hentVarbeidSted
	 */

	@Test
	public void testHentVarbeidSted_instnrOgAvdnrOgUnderavdOgGruppenrOgUtplukkOgMaksantall_200()
			throws Exception {
		// Arrange
		URL url = new URL(
				urlPrefix
						+ "hentVarbeidSted?instnr=185&avdnr=14&underavd=4&gruppenr=0&utplukk=nyeste&maksantall=20");
		HttpURLConnection connection = (HttpURLConnection) url.openConnection();
		connection.setRequestMethod("GET");

		// Act
		int res = connection.getResponseCode();
		getRespText(connection);
		connection.disconnect();

		// Assert
		Assert.assertEquals(200, res);
	}

	/*
	 * hentPresentasjon
	 */

	@Test
	public void testHentPresentasjon_personLopenr_200() throws Exception {
		// Arrange
		URL url = new URL(urlPrefix + "hentPresentasjon?lopenr=451");
		HttpURLConnection connection = (HttpURLConnection) url.openConnection();
		connection.setRequestMethod("GET");

		// Act
		int res = connection.getResponseCode();
		String resp = getRespText(connection);
		connection.disconnect();

		// Assert
		Assert.assertEquals(200, res);
		Assert.assertTrue(resp.contains("ola.nilsen"));
	}

	@Test
	public void testHentPresentasjon_enhetLopenr_200() throws Exception {
		// Arrange
		URL url = new URL(urlPrefix + "hentPresentasjon?lopenr=291893");
		HttpURLConnection connection = (HttpURLConnection) url.openConnection();
		connection.setRequestMethod("GET");

		// Act
		int res = connection.getResponseCode();
		String resp = getRespText(connection);
		connection.disconnect();

		// Assert
		Assert.assertEquals(200, res);
		Assert.assertTrue(resp.contains("Bioanalytics"));
	}

	@Test
	public void testHentPresentasjon_prosjektLopenr_200() throws Exception {
		// Arrange
		URL url = new URL(urlPrefix + "hentPresentasjon?lopenr=288000");
		HttpURLConnection connection = (HttpURLConnection) url.openConnection();
		connection.setRequestMethod("GET");

		// Act
		int res = connection.getResponseCode();
		String resp = getRespText(connection);
		connection.disconnect();

		// Assert
		Assert.assertEquals(200, res);
		Assert.assertTrue(resp.contains("Langtidsrehabilitering"));
	}

	@Test
	public void testHentPresentasjon_lopenr_404() throws Exception {
		// Arrange
		URL url = new URL(urlPrefix + "hentPresentasjon?lopenr=1");
		HttpURLConnection connection = (HttpURLConnection) url.openConnection();
		connection.setRequestMethod("GET");

		// Act
		int res = connection.getResponseCode();
		connection.disconnect();

		// Assert
		Assert.assertEquals(404, res);
	}

	/*
	 * hentPresentasjonSted
	 */

	@Test
	public void testHentPresentasjonSted_instnrOgType_200() throws Exception {
		// Arrange
		URL url = new URL(urlPrefix
				+ "hentPresentasjonSted?instnr=185&type=PROSJEKT");
		HttpURLConnection connection = (HttpURLConnection) url.openConnection();
		connection.setRequestMethod("GET");

		// Act
		int res = connection.getResponseCode();
		String resp = getRespText(connection);
		connection.disconnect();

		// Assert
		Assert.assertEquals(200, res);
		Assert.assertTrue(resp.contains("2007-01-08T13:58:19Z"));
	}

	/*
	 * hentPresentasjonBilde
	 */

	@Test
	public void testHentPresentasjonBilde_lopenr_200() throws Exception {
		// Arrange
		URL url = new URL(urlPrefix + "hentPresentasjonBilde?lopenr=49");
		HttpURLConnection connection = (HttpURLConnection) url.openConnection();
		connection.setRequestMethod("GET");

		// Act
		int res = connection.getResponseCode();
		getRespText(connection);
		connection.disconnect();

		// Assert
		Assert.assertEquals(200, res);
	}

	@Test
	public void testHentPresentasjonBilde_lopenr_404() throws Exception {
		// Arrange
		URL url = new URL(urlPrefix + "hentPresentasjonBilde?lopenr=50");
		HttpURLConnection connection = (HttpURLConnection) url.openConnection();
		connection.setRequestMethod("GET");

		// Act
		int res = connection.getResponseCode();
		connection.disconnect();

		// Assert
		Assert.assertEquals(404, res);
	}
}
