package no.usit.fridaws.test;

import java.util.ArrayList;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;

import junit.framework.TestCase;
import no.usit.fridaws.Xml;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

public class TestXml extends TestCase {

    public void testStringToXPath() {
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        DocumentBuilder docBuilder = null;
        try {
            docBuilder = dbf.newDocumentBuilder();
        } catch (ParserConfigurationException e) {
            fail("failed to create DocumentBuilder");
        }

        Document doc = docBuilder.newDocument();

        Element topEl = doc.createElement("topelement");
        doc.appendChild(topEl);
        Element innerEl = doc.createElement("innerelement");
        topEl.appendChild(innerEl);

        try {
            XPathExpression xp = Xml.stringToXPath("/");
            assertEquals((Node) xp.evaluate(doc, XPathConstants.NODE), doc);

            xp = Xml.stringToXPath("/topelement");
            assertEquals((Node) xp.evaluate(doc, XPathConstants.NODE), topEl);

            xp = Xml.stringToXPath("/topelement/innerelement");
            assertEquals((Node) xp.evaluate(doc, XPathConstants.NODE), innerEl);
        } catch (Exception e) {
            fail("Test failed due to " + e.getMessage());
        }
    }


    public void testJoinXml() {
        ArrayList<String> parts = new ArrayList<String>();
        parts.add("<?xml version=\"1.0\" encoding=\"UTF-8\"?>" + "<frida xmlns:xsi="
                  + "\"http://www.w3.org/2001/XMLSchema-instance\" "
                  + "xsi:noNamespaceSchemaLocation="
                  + "\"http://frida.usit.uio.no"
                  + "/eksport/forskningsresultater/fridaEksport.xsd\">"
                  + "<datoGenerert>2009-01-22</datoGenerert>"
                  + "<skjemaversjon>1.0</skjemaversjon>"
                  + "<forskningsresultat><A>AAAAAAA</A></forskningsresultat>"
                  + "</frida>");
        parts.add("<?xml version=\"1.0\" encoding=\"UTF-8\"?>" + "<frida xmlns:xsi="
                  + "\"http://www.w3.org/2001/XMLSchema-instance\" "
                  + "xsi:noNamespaceSchemaLocation="
                  + "\"http://frida.usit.uio.no"
                  + "/eksport/forskningsresultater/fridaEksport.xsd\">"
                  + "<datoGenerert>2009-01-22</datoGenerert>"
                  + "<skjemaversjon>1.0</skjemaversjon>"
                  + "<forskningsresultat><B>BBBB</B></forskningsresultat>"
                  + "</frida>");
        parts.add("<?xml version=\"1.0\" encoding=\"UTF-8\"?>" + "<frida xmlns:xsi="
                  + "\"http://www.w3.org/2001/XMLSchema-instance\" "
                  + "xsi:noNamespaceSchemaLocation="
                  + "\"http://frida.usit.uio.no"
                  + "/eksport/forskningsresultater/fridaEksport.xsd\">"
                  + "<datoGenerert>2009-01-22</datoGenerert>"
                  + "<skjemaversjon>1.0</skjemaversjon>"
                  + "<forskningsresultat><C>CC</C></forskningsresultat>"
                  + "</frida>");
        /*
        XPathExpression xp = null;
        try {
            xp = Xml.stringToXPath("/frida/forskningsresultat");
        } catch (Exception e) {
            fail("invalid xpath expression: " + e.getMessage());
        }
        */

        String res = null;
        try {
            // res = Xml.joinMedDato("2011-01-04", "frida", "forskningsresultat", parts);
        } catch (Throwable e) {
            String msg = e.getMessage();
            while ((e = e.getCause()) != null)
                msg += "\n\t" + e.getMessage();
            fail("join failed: " + msg);
        }

        String expectedRes = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" + "<frida xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xsi:noNamespaceSchemaLocation=\"frida\"><generert>2011-01-04</generert>"
                             + "<forskningsresultat><A>AAAAAAA</A></forskningsresultat>"
                             + "<forskningsresultat><B>BBBB</B></forskningsresultat>"
                             + "<forskningsresultat><C>CC</C></forskningsresultat>"
                             + "</frida>";

        assertEquals(res, expectedRes);
    }

}
